@@ -56,7 +56,7 @@ class Component:
 
         self.builder.component_ids[self.id] = self
 
-    def load_data(self, data, load_children=True):
+    def load_data(self, data):
         if self.input and data:
             try:
                 self.value = data[self.key]
@@ -66,39 +66,6 @@ class Component:
                 # TODO: Is this the right approach?
                 pass
 
-        if not load_children:
-            return
-
-        # (Input) nested components (e.g. datagrid, editgrid)
-        for component in self.raw.get('components', []):
-            # Only determine and load class if component type.
-            if 'type' in component:
-                component_obj = self.builder.get_component_object(component)
-                component_obj.load(self.child_component_owner, parent=self, data=data, all_data=self._all_data)
-
-        # TODO: This code is iffy and tries to be generic for unknown components.
-        # Maybe only call this (and the above) if not an input component?
-        # (Layout) nested components (e.g. columns, panels)
-        for k, vals in self.raw.copy().items():
-            if k == 'components':
-                continue # Already processed above, don't process subcomponents twice (#17)
-
-            if isinstance(vals, list):
-                for v in vals:
-                    if 'components' in v:
-                        for v_component in v['components']:
-                            v_component_obj = self.builder.get_component_object(v_component)
-                            v_component_obj.load(self.child_component_owner, parent=self, data=data, all_data=self._all_data)
-                    elif isinstance(v, list):
-                        # table component etc. which holds even deeper lists with components
-                        for list_v in v:
-                            if 'components' in list_v:
-                                for list_v_component in list_v.get('components'):
-                                    if list_v_component.get('type'):
-                                        list_v_component_obj = self.builder.get_component_object(list_v_component)
-                                        if list_v_component_obj.id not in self.builder.component_ids:
-                                            list_v_component_obj.load(self.child_component_owner, parent=self, data=data, all_data=self._all_data)
-
     @property
     def id(self):
         return self._id
@@ -234,7 +201,7 @@ class Component:
                     try:
                         context['row'] = self.component_owner.row
                     except AttributeError:
-                        pass # only datagrid rows have a "row" attribute
+                        pass  # only datagrid rows have a "row" attribute
                     return jsonLogic(cond['json'], context)
                 except ImportError:
                     logger = logging.getLogger(__name__)
@@ -261,6 +228,7 @@ class Component:
 
 # Basic
 
+
 class textfieldComponent(Component):
     pass
 
@@ -371,8 +339,6 @@ class radioComponent(Component):
     def value_label(self):
         comp = self.component_owner.input_components.get(self.key)
         builder_values = comp.raw.get('values')
-        value_label = {}
-
         for b_val in builder_values:
             if b_val['value'] == self.value:
                 if self.i18n.get(self.language):
@@ -417,9 +383,6 @@ class addressComponent(Component):
     # XXX other providers not analysed and implemented yet.
     PROVIDER_GOOGLE = 'google'
 
-    def load_data(self, data):
-        super(addressComponent, self).load_data(data, load_children=False)
-
     def _address_google(self, get_type, notation='long_name'):
         comps = self.value.get('address_components')
         if not comps:
@@ -546,7 +509,7 @@ class datetimeComponent(Component):
                 py_dt_format = py_dt_format.replace(formio, py)
                 done = True
 
-        #day
+        # day
         done = False
         for formio, py in mapping['day'].items():
             if not done and formio in formio_dt_format:
@@ -729,7 +692,6 @@ class surveyComponent(Component):
             question_dict = {'question_value': question['value'], 'question_label': question_label, 'values': []}
 
             # value
-            values = []
             for b_val in builder_values:
                 if self.i18n.get(self.language):
                     val_label = self.i18n[self.language].get(b_val['label'], b_val['label'])
@@ -739,7 +701,7 @@ class surveyComponent(Component):
                 value = {
                     'label': val_label,
                     'value': b_val['value'],
-                    'checked': False # default as fallback (if new values in builder)
+                    'checked': False  # default as fallback (if new values in builder)
                 }
 
                 if self.value.get(question['value']):
@@ -751,6 +713,7 @@ class surveyComponent(Component):
             grid.append(question_dict)
         return grid
 
+
 class signatureComponent(Component):
     pass
 
@@ -764,6 +727,7 @@ class htmlelementComponent(Component):
         html = '<%s>%s</%s>' % (self.raw['tag'], self.raw['content'], self.raw['tag'])
         return html
 
+
 class contentComponent(Component):
     pass
 
@@ -774,6 +738,14 @@ class layoutComponentBase(Component):
 
 class columnsComponent(layoutComponentBase):
 
+    def load_data(self, data):
+        for column in self.raw['columns']:
+            for component in column['components']:
+                # Only determine and load class if component type.
+                if 'type' in component:
+                    component_obj = self.builder.get_component_object(component)
+                    component_obj.load(self.child_component_owner, parent=self, data=data, all_data=self._all_data)
+
     @property
     def rows(self):
         rows = []
@@ -822,7 +794,6 @@ class columnsComponent(layoutComponentBase):
             rows.append(row)
         return rows
 
-
     def render(self):
         html_rows = []
         for row in self.rows:
@@ -846,6 +817,13 @@ class fieldsetComponent(layoutComponentBase):
 
 class panelComponent(layoutComponentBase):
 
+    def load_data(self, data):
+        for component in self.raw.get('components', []):
+            # Only determine and load class if component type.
+            if 'type' in component:
+                component_obj = self.builder.get_component_object(component)
+                component_obj.load(self.child_component_owner, parent=self, data=data, all_data=self._all_data)
+
     @property
     def title(self):
         title = self.raw.get('title')
@@ -859,38 +837,46 @@ class panelComponent(layoutComponentBase):
 
 
 class tableComponent(layoutComponentBase):
+    def __init__(self, raw, builder, **kwargs):
+        self.rows = []
+        super().__init__(raw, builder, **kwargs)
 
-    @property
-    def rows(self):
-        rows = []
-        for row in self.raw['rows']:
-            row_components = []
+    def load_data(self, data):
+        self.rows = []
 
-            for cols in row:
-                for col_comp in cols['components']:
-                    for key, comp in self.components.items():
-                        if col_comp['id'] == comp.id:
-                            row_components.append(comp)
-            rows.append(row_components)
-        return rows
+        for data_row in self.raw.get('rows', []):
+            row = []
+
+            for col in data_row:
+                components = []
+                for component in col['components']:
+                    # Only determine and load class if component type.
+                    if 'type' in component:
+                        component_obj = self.builder.get_component_object(component)
+                        component_obj.load(self.child_component_owner, parent=self, data=data, all_data=self._all_data)
+                        components.append(component_obj)
+
+                row.append({'column': col, 'components': components})
+
+            self.rows.append(row)
 
 
 class tabsComponent(layoutComponentBase):
 
-    @property
-    def tabs(self):
-        tabs = []
-        for tab in self.raw['components']:
-            add_tab = {
-                'tab': tab,
-                'components': []
-            }
-            for comp in tab['components']:
-                for key, comp in self.components.items():
-                    if comp['key'] == comp.key:
-                        add_tab['components'].append(comp[1])
-            tabs.append(add_tab)
-        return tabs
+    def load_data(self, data):
+        self.tabs = []
+
+        for data_tab in self.raw.get('components', []):
+            tab = {'tab': data_tab, 'components': []}
+
+            for component in data_tab['components']:
+                # Only determine and load class if component type.
+                if 'type' in component:
+                    component_obj = self.builder.get_component_object(component)
+                    component_obj.load(self.child_component_owner, parent=self, data=data, all_data=self._all_data)
+                    tab['components'].append(component_obj)
+
+            self.tabs.append(tab)
 
 
 # Data components
@@ -911,7 +897,6 @@ class datagridComponent(Component):
             self.form = datagrid.form
             self.row = data
             self.html_component = ''
-
             datagrid.create_component_objects(self, data)
 
         def render(self):
@@ -922,10 +907,8 @@ class datagridComponent(Component):
                 else:
                     component.html_component = ''
                 html_components.append('<td>'+component.html_component+'</td>')
-
             self.html_component = '<tr>'+(''.join(html_components))+'</tr>'
 
-
     def __init__(self, raw, builder, **kwargs):
         # TODO when adding other data/grid components, create new
         # dataComponent class these can inherit from.
@@ -990,7 +973,6 @@ class datagridComponent(Component):
         # property of its owner.
         return True
 
-
     @property
     def child_component_owner(self):
         return self
@@ -1025,7 +1007,6 @@ class fileComponent(Component):
         if self.storage == 'url':
             res = ''
             for val in self.form.get('value'):
-                name = val.get('name')
                 url = val.get('url')
                 res += base64_encode_url(url)
             return res
